#include "ArduinoParticipant.h"

#if !defined(NO_STD)
#include <iostream>
#endif

#if defined(ARDUINO)
#if defined(ARDUINO_ARCH_ESP8266)
#include <ESP8266WiFi.h>

#elif defined(ESP32)
#include <WiFi.h>

#elif defined(UNO_R4)
#include <WiFi.h>

#elif defined(ARDUINO_ARCH_RP2040)  // not functional, for future use
#include <WifiNINA.h>

#endif
#endif

namespace RoboidControl {
namespace Arduino {

#if defined(ARDUINO) && defined(HAS_WIFI)
WiFiUDP* udp;
#endif

void ParticipantUDP::Setup() {
#if defined(ARDUINO) && defined(HAS_WIFI)
  GetBroadcastAddress();

#if defined(UNO_R4)
  if (WiFi.status() == WL_NO_MODULE) {
#if !defined(NO_STD)
    std::cout << "No network available!\n";
#endif
    return;
  }
#else
  if (WiFi.isConnected() == false) {
    std::cout << "No network available!\n";
    return;
  }
#endif

  udp = new WiFiUDP();
  udp->begin(this->port);

#if !defined(NO_STD)
  std::cout << "Wifi sync started local " << this->port;
  if (this->remoteSite != nullptr)
    std::cout << ", remote " << this->remoteSite->ipAddress << ":"
              << this->remoteSite->port << "\n";
#endif
#endif
}

void ParticipantUDP::GetBroadcastAddress() {
#if defined(ARDUINO) && defined(HAS_WIFI)
  IPAddress broadcastAddress = WiFi.localIP();
  broadcastAddress[3] = 255;
  String broadcastIpString = broadcastAddress.toString();
  this->broadcastIpAddress = new char[broadcastIpString.length() + 1];
  broadcastIpString.toCharArray(this->broadcastIpAddress,
                                broadcastIpString.length() + 1);
#if !defined(NO_STD)
  std::cout << "Broadcast address: " << broadcastIpAddress << "\n";
#endif
#endif
}

void ParticipantUDP::Receive() {
#if defined(ARDUINO) && defined(HAS_WIFI)
  int packetSize = udp->parsePacket();
  while (packetSize > 0) {
    udp->read(buffer, packetSize);

    String senderAddress = udp->remoteIP().toString();
    char sender_ipAddress[16];
    senderAddress.toCharArray(sender_ipAddress, 16);
    unsigned int sender_port = udp->remotePort();

    ReceiveData(packetSize, sender_ipAddress, sender_port);
    packetSize = udp->parsePacket();
  }
#endif
}

bool ParticipantUDP::Send(Participant* remoteParticipant, int bufferSize) {
#if defined(ARDUINO) && defined(HAS_WIFI)
  // std::cout << "Sending to:\n " << remoteParticipant->ipAddress << ":"
  // << remoteParticipant->port << "\n";

  int n = 0;
  int r = 0;
  do {
    if (n > 0) {
#if !defined(NO_STD)
      std::cout << "Retry sending\n";
#endif
      delay(10);
    }
    n++;

    udp->beginPacket(remoteParticipant->ipAddress, remoteParticipant->port);
    udp->write((unsigned char*)buffer, bufferSize);
    r = udp->endPacket();
    // On an Uno R4 WiFi, endPacket blocks for 10 seconds the first time
    // It is not cleary yet why
  } while (r == 0 && n < 10);

#endif
  return true;
}

bool ParticipantUDP::Publish(IMessage* msg) {
#if defined(ARDUINO) && defined(HAS_WIFI)
  int bufferSize = msg->Serialize((char*)this->buffer);
  if (bufferSize <= 0)
    return true;

  udp->beginPacket(this->broadcastIpAddress, this->port);
  udp->write((unsigned char*)buffer, bufferSize);
  udp->endPacket();

  //   std::cout << "Publish to " << this->broadcastIpAddress << ":"
  //             << this->remotePort << "\n";
#endif
  return true;
};

}  // namespace Arduino
}  // namespace RoboidControl
